module EasySwig

	# Generation Settings
  class Config
    # [String] Working directory. Defaults to current working directory
    attr_accessor :default_dir
  	# [String] Name of the module. Used to name generated resources. Defaults to the name of the default (working) directory
    attr_accessor :module_name
    # [String] Directory generated by Doxygen (which include a /xml subdirectory for all XML files. Defaults to: path/to/working/dir/easy-swig/doxygen
    attr_accessor :doxy_dir
    # [String] Directory for your project's header files. Defaults to: path/to/working/dir/include
    attr_accessor :headers_dir
    # [String] CSV configuration file input. Defaults to: path/to/working/dir/api.csv
    attr_accessor :csv_file
    # [String] Output directory for SWIG generated files. Defaults to : path/to/working/dir/easy-swig/swig
    attr_accessor :output_dir
    # [String] Output directory for Easy-Swig generated .i file(s) aka. 'target'. Defaults to path/to/working/dir/easy-swig/generate
    attr_accessor :generate_dir
    # [String] Instructs Doxygen to also generate HTML documentation for the header files. Defaults to nil (false)
    attr_accessor :html
    # [String] Target language for SWIG. Defaults to 'csharp' (C#)
    attr_accessor :lang
    # [String] Custom .i file to be appended at the beginning of the generated .i target. Used for advanced configuration
    # 	Defaults to path/to/working/dir/custom_config.i
    attr_accessor :custom_file
    # [String] Target .i file as input for SWIG. If a directory SWIG will be invoked once for every found .i file
    # 	Defaults to path/to/working/dir/easy-swig/generate/<MODULE_NAME>.i
    attr_accessor :target_file
    # [Array<String>] System include directories. Defaults to Linux include directory: ["/usr/include"]
    attr_accessor :includes_dir
    # [String] Special support for the Standard Template Library. Values 'YES'/'NO' (defaults to 'YES') 
    attr_accessor :stl_support
    
    # Creates a new Configutation using the default values for a given directory
    # @param [String] default_dir Working directory
    def initialize(default_dir = nil)
    	@default_dir = default_dir
    	set_defaults
    end

    def set_defaults
      @default_dir ||= Dir.getwd
      @module_name ||= File.basename(@default_dir)
      @output_dir ||= "#{@default_dir}/easy-swig/swig"
      @doxy_dir ||= "#{@default_dir}/easy-swig/doxygen"
      @generate_dir ||= "#{@default_dir}/easy-swig/generate"
      @headers_dir ||= "#{@default_dir}/include"
      @csv_file ||= "#{@default_dir}/api.csv"
      @custom_file ||= "#{@default_dir}/custom_config.i"
      @lang ||= "csharp"
      @stl_support ||= 'YES'
      @target_file ||= "#{@default_dir}/easy-swig/generate/#{@lang}/#{@module_name}.i"
      @includes_dir ||= ["/usr/include"]
    end
  end
end